@@ -2,11 +2,16 @@ import copy
 import importlib
 import json
 from pathlib import Path
-from typing import Dict, List, Type
+from typing import Dict, List, Type, Union
 
 import yaml
 
-from .builders import ModelBuilderStack, OutputBuilder, OutputBuilderComponent, ReplaceElement
+from .builders import (
+    ModelBuilderStack,
+    OutputBuilder,
+    OutputBuilderComponent,
+    ReplaceElement,
+)
 from .fs import AbstractFileSystem, FileSystem
 from .model_preprocessors import ModelPreprocessor
 from .outputs import OutputBase
@@ -21,27 +26,27 @@ class ModelBuilder:
     Processes a model file and generates/updates sources for the model
     """
 
-    model_preprocessor_classes: List[type(ModelPreprocessor)]
+    model_preprocessor_classes: List[Type[ModelPreprocessor]]
     """
     Model preprocessor classes that are called after schema is loaded and before it is processed
     """
 
-    output_classes: List[type(OutputBase)]
+    output_classes: List[Type[OutputBase]]
     """
     Mapping between output type and its handler class
     """
 
-    filtered_output_classes: Dict[str, type(OutputBase)]
+    filtered_output_classes: Dict[str, Type[OutputBase]]
     """
     Filtered output classes by settings.plugins.disabled|enabled
     """
 
-    output_builder_classes: List[type(OutputBuilder)]
+    output_builder_classes: List[Type[OutputBuilder]]
     """
     A list of extension classes to be used in build. 
     """
 
-    property_preprocessor_classes: List[type(PropertyPreprocessor)]
+    property_preprocessor_classes: List[Type[PropertyPreprocessor]]
     """
     Processor classes (called before and after file builder is called)
     """
@@ -68,15 +73,26 @@ class ModelBuilder:
 
     filesystem: AbstractFileSystem
 
+    overwrite: bool
+    """
+    If true, overwrite already existing files. If false, perform merge
+    """
+
+    conflict_resolver: ConflictResolver
+    """
+    Resolver for conflicts
+    """
+
     def __init__(
-            self,
-            outputs: List[type(OutputBase)] = (),
-            output_builders: List[type(OutputBuilder)] = (),
-            property_preprocessors: List[type(PropertyPreprocessor)] = (),
-            model_preprocessors: List[type(ModelPreprocessor)] = (),
-            output_builder_components: Dict[str, List[type(OutputBuilderComponent)]] = None,
-            included_validation_schemas=None,
-            filesystem=FileSystem(),
+        self,
+        outputs: List[Type[OutputBase]] = (),
+        output_builders: List[Type[OutputBuilder]] = (),
+        property_preprocessors: List[Type[PropertyPreprocessor]] = (),
+        model_preprocessors: List[Type[ModelPreprocessor]] = (),
+        output_builder_components: Dict[str, List[Type[OutputBuilderComponent]]] = None,
+        filesystem=FileSystem(),
+        conflict_resolver: ConflictResolver = None,
+        overwrite=False,
     ):
         """
         Initializes the builder
@@ -88,9 +104,10 @@ class ModelBuilder:
         self.output_builder_classes = [*output_builders]
         for o in outputs:
             assert o.TYPE, f"output_type not set up on class {o}"
-        self.output_classes = [*outputs]
-        self.property_preprocessor_classes = [*property_preprocessors]
-        self.model_preprocessor_classes = [*model_preprocessors]
+        self.output_classes = [*(outputs or [])]
+        self.outputs = {}
+        self.property_preprocessor_classes = [*(property_preprocessors or [])]
+        self.model_preprocessor_classes = [*(model_preprocessors or [])]
         self.filtered_output_classes = {o.TYPE: o for o in self.output_classes}
         if output_builder_components:
             self.output_builder_components = {
@@ -100,10 +117,11 @@ class ModelBuilder:
         else:
             self.output_builder_components = {}
         self.filesystem = filesystem
-        self.included_validation_schemas = included_validation_schemas or []
-        self.skip_schema_validation = False     # set to True in some tests
+        self.skip_schema_validation = False  # set to True in some tests
+        self.conflict_resolver = conflict_resolver
+        self.overwrite = overwrite
 
-    def get_output(self, output_type: str, path: str | Path):
+    def get_output(self, output_type: str, path: Union[str, Path]):
         """
         Given a path, instantiate file builder on the path with the given output type
         and return it. If the builder on the path has already been requested, return
@@ -130,7 +148,11 @@ class ModelBuilder:
         return self.output_builder_components.get(output_builder_type, ())
 
     # main entry point
-    def build(self, model: ModelSchema, output_dir: str | Path, resolver: ConflictResolver = None):
+    def build(
+        self,
+        model: ModelSchema,
+        output_dir: Union[str, Path],
+    ):
         """
         compile the schema to output directory
 
@@ -138,38 +160,66 @@ class ModelBuilder:
         :param output_dir:  output directory where to put generated files
         :return:            the outputs (self.outputs)
         """
-        self.conflict_resolver = resolver
+        if self.overwrite:
+            if not hasattr(self.filesystem, "overwrite"):
+                raise AttributeError(
+                    f"Filesystem of type {type(self.filesystem)} does not support overwrite"
+                )
+            self.filesystem.overwrite = True
+
         self.set_schema(model)
-        self.filtered_output_classes = {o.TYPE: o for o in self._filter_classes(self.output_classes, "output")}
+        self.filtered_output_classes = {
+            o.TYPE: o for o in self._filter_classes(self.output_classes, "output")
+        }
         self.output_dir = Path(output_dir).absolute()  # noqa
         self.outputs = {}
 
-        if not self.skip_schema_validation:
-            validate_model(model, self.included_validation_schemas)
+        self._validate_model(model)
 
-        for model_preprocessor in self._filter_classes(self.model_preprocessor_classes, "model"):
-            model_preprocessor(self).transform(model, model.settings)
+        self._run_model_preprocessors(model)
 
-        if not self.skip_schema_validation:
-            validate_model(model, self.included_validation_schemas)
+        self._validate_model(model)
 
         # noinspection PyTypeChecker
         property_preprocessors: List[PropertyPreprocessor] = [
-            e(self) for e in self._filter_classes(self.property_preprocessor_classes, "property")
+            e(self)
+            for e in self._filter_classes(
+                self.property_preprocessor_classes, "property"
+            )
         ]
 
+        self._run_output_builders(model, property_preprocessors)
+
+        self._save_outputs()
+
+        return self.outputs
+
+    def _run_output_builders(self, model, property_preprocessors):
         output_builder_class: Type[OutputBuilder]
-        for output_builder_class in self._filter_classes(self.output_builder_classes, "builder"):
-            output_builder = output_builder_class(builder=self, property_preprocessors=property_preprocessors)
+        for output_builder_class in self._filter_classes(
+            self.output_builder_classes, "builder"
+        ):
+            output_builder = output_builder_class(
+                builder=self, property_preprocessors=property_preprocessors
+            )
             output_builder.build(model)
 
+    def _run_model_preprocessors(self, model):
+        for model_preprocessor in self._filter_classes(
+            self.model_preprocessor_classes, "model"
+        ):
+            model_preprocessor(self).transform(model, model.settings)
+
+    def _validate_model(self, model):
+        if not self.skip_schema_validation:
+            validate_model(model)
+
+    def _save_outputs(self):
         for output in sorted(self.outputs.values(), key=lambda x: x.path):
             output.finish()
             if output.executable:
                 self.filesystem.make_executable(output.path)
 
-        return self.outputs
-
     def set_schema(self, schema):
         self.schema = schema
         self.settings = schema.settings
@@ -177,9 +227,12 @@ class ModelBuilder:
     # private methods
 
     def _filter_classes(self, classes: List[Type[object]], plugin_type):
-        if "plugins" not in self.schema.schema or plugin_type not in self.schema.schema.plugins:
+        if (
+            "plugins" not in self.schema.current_model
+            or plugin_type not in self.schema.current_model.plugins
+        ):
             return classes
-        plugin_config = self.schema.schema.plugins[plugin_type]
+        plugin_config = self.schema.current_model.plugins[plugin_type]
 
         disabled = plugin_config.get("disable", [])
         enabled = plugin_config.get("enable", [])
