@@ -1,4 +1,6 @@
 from .invenio_base import InvenioBaseClassPythonBuilder
+from oarepo_model_builder.outputs.python import PythonOutput
+from oarepo_model_builder.utils.hyphen_munch import HyphenMunch
 
 
 class InvenioRecordServiceBuilder(InvenioBaseClassPythonBuilder):
